#ifndef DLS_H
#define DLS_H

/* compression mode */
#define MODE_USE_FLAG       0
#define MODE_COMPRESS_ALL   1
#define MODE_COMPRESS_NONE  2

#define ChunkName(a,b,c,d)(	    	\
				(a & 0xFF)	        \
			+	((b & 0xFF) << 8)	\
			+	((c & 0xFF) << 16)	\
			+	((d & 0xFF) << 24))

#define RIFF    ChunkName('R','I','F','F')
#define DLS     ChunkName('D','L','S',' ') 
#define LIST    ChunkName('L','I','S','T')
#define COLH    ChunkName('c','o','l','h')
#define VERS    ChunkName('v','e','r','s')
#define LINS    ChunkName('l','i','n','s')
#define INS     ChunkName('i','n','s',' ')
#define INSH    ChunkName('i','n','s','h')
#define LRGN    ChunkName('l','r','g','n')
#define RGN     ChunkName('r','g','n',' ')
#define RGNH    ChunkName('r','g','n','h')
#define WSMP    ChunkName('w','s','m','p')
#define WLNK    ChunkName('w','l','n','k')
#define LART    ChunkName('l','a','r','t')
#define ART1    ChunkName('a','r','t','1')
#define DATA    ChunkName('d','a','t','a')
#define PTBL    ChunkName('p','t','b','l')
#define DLID    ChunkName('d','l','i','d')
#define WVPL    ChunkName('w','v','p','l')
#define WAVE    ChunkName('w','a','v','e')
#define WAVU    ChunkName('w','a','v','u')
#define FMT     ChunkName('f','m','t',' ')
#define INFO    ChunkName('I','N','F','O')
#define IARL    ChunkName('I','A','R','L')
#define IART    ChunkName('I','A','R','T')
#define ICMS    ChunkName('I','C','M','S')
#define ICMT    ChunkName('I','C','M','T')
#define ICOP    ChunkName('I','C','O','P')
#define ICRD    ChunkName('I','C','R','D')
#define IENG    ChunkName('I','E','N','G')
#define IGNR    ChunkName('I','G','N','R')
#define IKEY    ChunkName('I','K','E','Y')
#define IMED    ChunkName('I','M','E','D')
#define INAM    ChunkName('I','N','A','M')
#define IPRD    ChunkName('I','P','R','D')
#define ISBJ    ChunkName('I','S','B','J')
#define ISFT    ChunkName('I','S','F','T')
#define ISRC    ChunkName('I','S','R','C')
#define ISRF    ChunkName('I','S','R','F')
#define ITCH    ChunkName('I','T','C','H')
#define SMPL	ChunkName('s','m','p','l')
#define ISTD    ChunkName('I','S','T','D')

#define	CONN_SRC_NONE				0x0000	
#define CONN_SRC_LFO				0x0001
#define CONN_SRC_KEYONVELOCITY		0x0002
#define CONN_SRC_KEYNUMBER			0x0003
#define CONN_SRC_EG1				0x0004
#define CONN_SRC_EG2				0x0005
#define CONN_SRC_PITCHWHEEL			0x0006

#define CONN_SRC_CC1				0x0081	/*	mod wheel       */
#define CONN_SRC_CC7				0x0087	/*	channel volume  */
#define CONN_SRC_CC10				0x008A	/*	pan             */
#define CONN_SRC_CC11				0x008B	/*	expression      */

#define CONN_DST_NONE				0x0000
#define CONN_DST_ATTENUATION		0x0001
#define CONN_DST_RESERVED			0x0002
#define CONN_DST_PITCH				0x0003
#define CONN_DST_PAN				0x0004

#define CONN_DST_LFO_FREQUENCY		0x0104
#define CONN_DST_LFO_STARTDELAY		0x0105

#define CONN_DST_EG1_ATTACKTIME		0x0206
#define CONN_DST_EG1_DECAYTIME		0x0207
#define CONN_DST_EG1_RESERVED		0x0208
#define CONN_DST_EG1_RELEASETIME	0x0209
#define CONN_DST_EG1_SUSTAINLEVEL	0x020A	

#define CONN_DST_EG2_ATTACKTIME		0x030A
#define CONN_DST_EG2_DECAYTIME		0x030B
#define CONN_DST_EG2_RESERVED		0x030C
#define CONN_DST_EG2_RELEASETIME	0x030D
#define CONN_DST_EG2_SUSTAINLEVEL	0x030E

#define CONN_TRN_NONE				0x0000
#define CONN_TRN_CONCAVE			0x0001

#define WT_FORMAT_ADPCM             0 
#define WT_FORMAT_PCM16             1
#define WT_FORMAT_PCM8              2

typedef struct _wt_region
{
    U8  unity_note;
    U8  key_group;
    
    S16 fine_tune;
    S32 attn;
    
    U32 loop_start;
    U32 loop_length;

    U32 articulation_index;  /* articulation index to reference */
    U32 sample_index;        /* sample index to reference       */

} WT_REGION_TYPE;

typedef struct _wt_sample
{
    U16 format;      /* ADPCM, PCM16, PCM8                          */
    U16 sample_rate; /* Hz                                          */
    U32 offset;      /* offset in bytes from beginning of PCM file  */
    U32 length;      /* length of sample in bytes                   */
    U16 adpcm_index; /* ADPCM index to reference if in ADPCM mode   */
} WT_SAMPLE_TYPE;

typedef struct _wt_inst   /* insturment */
{
    U16 key_region[128];
} WT_INST_TYPE;

    
typedef struct _wt_art    /* translated DLS articulartion block */
{
    /* LFO */
    S32 lfo_freq;
    S32 lfo_delay;
    S32 lfo_atten;
    S32 lfo_pitch;
    S32 lfo_mod2atten;
    S32 lfo_mod2pitch;

    /* EG1 */
    S32 eg1_attack;
    S32 eg1_decay;
    S32 eg1_sustain;
    S32 eg1_release;
    S32 eg1_vel2attack;
    S32 eg1_key2decay;

    /* EG2 */
    S32 eg2_attack;
    S32 eg2_decay;
    S32 eg2_sustain;
    S32 eg2_release;
    S32 eg2_vel2attack;
    S32 eg2_key2decay;
    S32 eg2_pitch;

    /* pan */
    S32 pan;

} WT_ART_TYPE;

typedef struct _wt_adpcm
{
    /* values to program at start */
    U16     coef[16];           /*  coef table a1[0],a2[0],a1[1],a2[1]....                   */
    U16     gain;               /*  gain to be applied (0 for ADPCM, 0x0800 for PCM8/16)     */
    U16     pred_scale;         /*  predictor / scale combination (nibbles, as in hardware)  */
    U16     yn1;                /*  y[n - 1]                                                 */
    U16     yn2;                /*  y[n - 2]                                                 */

    /* loop context */
    U16     loop_pred_scale;    /*  predictor / scale combination (nibbles, as in hardware) */
    U16     loop_yn1;           /*  y[n - 1]                                                */
    U16     loop_yn2;           /*  y[n - 2]                                                */

} WT_ADPCM_TYPE;

typedef struct _wt_file_header
{
    U32 offset_percussive_inst;
    U32 offset_melodic_inst;
    U32 offset_regions;
    U32 offset_articulations;
    U32 offset_samples;
    U32 offset_adpcm_context;
    
} WT_FILE_HEADER_TYPE;


/*
================================================================
=
= Function: dls_read_file
=
= Description:
=
=    convert the dls file into wave header and pcm file 
=
================================================================ 
*/

extern void dls_read_file(FILE *dls_file, FILE *wt_file, FILE *pcm_file, S32 mode);

#endif
