/******************************************************************************
**  dls converter
**  --------------------------------------------------------------------------
**  
**  convert dls file into wave table header and a pcm / adpcm file.
**  
**  Author: s.taffs
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2 of the License,
** or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.
**
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "types.h"
#include "dls.h"

/* macro defines */

#define REQ_COMMAND_LINE_PRAM   3
#define VERSION                 "0.3V\n"
#define USAGE                   "DLS converter version: " VERSION \
								"Usage example: dls file_in.dls -options\n" \
								"\n    -d   use DLS F_WSMP_NO_COMPRESSION flag (default)\n" \
								"    -a   ADPCM compress all samples\n"	\
								"    -p   PCM encode all samples\n"								 			
#define FILE_NAME_IN            1

/* external variables */

/* None */

/* local variables */

/* None */

/* global variables */

/* None */

/* local prototypes */

/* None */

/*
================================================================
=
= Function: main
=
= Description:
=
=    convert dls file into wave table header and pcm files 
=
================================================================ 
*/

int main(int argc, char *argv[]) 
{
	FILE *file_ptr_in = NULL;
	FILE *file_ptr_out_wt = NULL;
	FILE *file_ptr_out_pcm = NULL;
	U64 file_size;
	char file_name[1024];
	char *char_ptr = NULL;
	S32 mode = MODE_USE_FLAG;
	
	if(argc < REQ_COMMAND_LINE_PRAM)
    {
        printf("Incorrect number of args \n");
        printf(USAGE);
        return EXIT_FAILURE; 
    }
    
    /* set compression type */
    if(_strcmpi(argv[2], "-d") == 0)
	{
		mode = MODE_USE_FLAG;
	}
    else if(_strcmpi(argv[2], "-a") == 0)
	{
		mode = MODE_COMPRESS_ALL;
	}
	else if(_strcmpi(argv[2], "-p") == 0)
	{
		mode = MODE_COMPRESS_NONE;
	}
    
    file_ptr_in = fopen(argv[FILE_NAME_IN],"rb");
    
    if(file_ptr_in == NULL)
    {
        printf("cant open file %s \n",argv[FILE_NAME_IN]);
        return EXIT_FAILURE;
    }
    
    fseek(file_ptr_in,0, SEEK_END);
    file_size = ftell(file_ptr_in);
    rewind(file_ptr_in);
    
    /* check to make sure file size is greater then zero */
    if(0 == file_size)
    {
    	fclose(file_ptr_in);
    	printf("file %s is empty !\n",argv[FILE_NAME_IN]);
        return EXIT_FAILURE;
	}
    
    /* copy the input file name across */
    strcpy(file_name, argv[FILE_NAME_IN]);
    
    /* find the end of the file name */
    char_ptr = strchr(file_name, '.');
    
    if(char_ptr == NULL)
    {
        printf("failed to open wt output file \n");
        fclose(file_ptr_in);
        return EXIT_FAILURE;
	}
	
	/* add wt extention to file name */
    char_ptr++;
    *char_ptr = 'w';
    char_ptr++;
    *char_ptr = 't';
    char_ptr++;
    *char_ptr = 0;
	
	file_ptr_out_wt = fopen(file_name,"wb");
	
	if(file_ptr_out_wt == NULL)
    {
        printf("failed to open output file %s \n",file_name);
        fclose(file_ptr_in);
        return EXIT_FAILURE;
    }
    
    /* roll back 2 chars */
    char_ptr -= 2;
    
    /* add pcm extention to file name */
    *char_ptr = 'p';
    char_ptr++;
    *char_ptr = 'c';
    char_ptr++;
    *char_ptr = 'm';
    char_ptr++;
    *char_ptr = 0;
    
    file_ptr_out_pcm = fopen(file_name,"wb");
	
	if(file_ptr_out_pcm == NULL)
    {
        printf("failed to open output file %s \n",file_name);
        fclose(file_ptr_in);
        fclose(file_ptr_out_wt);
        return EXIT_FAILURE;
    }
    
    /* convert the dls file into wave header and pcm file */
    dls_read_file(file_ptr_in, file_ptr_out_wt, file_ptr_out_pcm, mode);
    
    fclose(file_ptr_in);
    fclose(file_ptr_out_wt);
    fclose(file_ptr_out_pcm);
	
	return EXIT_SUCCESS;
}
